#ifndef KDK_DISK_ENCRYPT_H
#define KDK_DISK_ENCRYPT_H
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

typedef enum _crypt_disk_format
{
    DISK_FORMAT = 0,
    DISK_NO_FORMAT = 1,
} crypt_disk_format;

#ifdef __cplusplus
extern "C"
{
#endif
    /* 
启用磁盘加密  使未加密的磁盘变成加密密文磁盘
    输入参数
                device: 设备分区例如：/dev/sdb1
                format: 是否格式化
        password： 密码
        passwortlen： 密码长度
    返回值
        成功返回0
        失败返回非0
*/
    int crypt_disk_encrypt_device(char *device, crypt_disk_format format, char *password, size_t passwordLen);

    /* 
停用磁盘加密，使已经加密的磁盘变成未加密的明文磁盘
    输入参数
                device: 设备分区例如：分区uuid或者/dev/sdb1
        password： 密码
        passwortlen： 密码长度
    返回值
        成功返回0
        失败返回非0
*/
    int crypt_disk_decrypt_device(char *device, char *password, size_t passwortlen);

    /* 
解锁未打开的加密磁盘，使加密磁盘能够进行读写
    输入参数
                device: 设备分区例如：/dev/sdb1
        password： 密码
        passwortlen： 密码长度
    返回值
        成功返回0
        失败返回非0
*/
    int crypt_disk_open_encrypted_device(char *device, char *password, size_t passwortlen);

    /* 
锁定打开的加密磁盘，使加密磁盘不能够进行读写
    输入参数
                device: 设备分区例如：/dev/sdb1
        password： 密码
        passwortlen： 密码长度
    返回值
        成功返回0
        失败返回非0
*/
    int crypt_disk_close_encrypted_device(char *device);

    /* 
更改密钥
    输入参数
                device: 设备分区例如：/dev/sdb1
        password： 旧密码
        passwortlen： 旧密码长度
        newpassword： 新密码
        newpasswortlen：新密码长度
    返回值
        成功返回0
        失败返回非0
*/
    int crypt_disk_change_password(char *device, char *password, size_t passwortlen, char *newpassword, size_t newpasswortlen);

    /* 
密码重置
    输入参数
                device: 设备分区例如：/dev/sdb1
        FilePath： 恢复密钥文件路径
        newpassword： 新密码
        newpasswortlen：新密码长度
    返回值
        成功返回0
        失败返回非0
*/
    int crypt_disk_reset_password(char *device, char *FilePath, char *newpassword, size_t newpasswordlen);

    /* 
判断磁盘是否解锁
    输入参数
                device: 设备分区例如：分区uuid或者/dev/sdb1
    返回值
        0 表示已经解锁
        1 表示未解锁
        其它表示错误
*/
    int crypt_disk_is_opened(char *device);

    /* 
判断磁盘是否可以保留数据加密
   输入参数
                device: 设备分区例如：/dev/sdb1
    返回值
        0 表示可以保留数据加密
        1 表示不可以保留数据加密
        其它表示错误
*/
    int crypt_disk_can_not_formatted(char *device);

    /* 
判断磁盘是否加密
    输入参数
                device: 设备分区例如：/dev/sdb1
    返回值
        0 表示已经加密
        1 表示未加密
        其它表示错误
*/
    int crypt_disk_is_encrypted(char *device);

    /* 
导出恢复密钥
    输入参数
                device: 设备分区例如：/dev/sdb1
                path:备份密钥文件路径
        oldpassword： 旧密码
        oldpasswortlen：旧密码长度
    返回值
        成功返回0
        失败返回非0
*/
    int crypt_disk_export_backup_key(char *device, char *path, char *oldpassword, size_t oldpasswordlen);

    /*
输入参数：device 设备路径
返回参数：path设备默认的备份密钥路径，需传入大于128字节的字符数组，例如char path[129]={0};
返回值：
        0 表示成功
       其它表示错误
*/
    int crypt_disk_default_backup_path(char *device, char *path);

    /*
返回进度值,支持保留数据加密和停用加密
输入参数：无
返回值：
	float类型的小于等于1的值
*/
    float crypt_get_progress_bar(char *device);
    /* 
导出口令
输入参数
		path: 恢复密钥文件
输出参数
	    password：口令
	    passwdLen：口令长度
返回值
        0 表示成功
        其它表示错误,具体参照错误列表
*/
    int crypt_disk_export_password(char *FilePath, char *password, int *passwdLen);

/* 
获取挂载点
输入参数
		volName: 设备路径，例如/dev/sdb1
输出参数
	    mountpoint：挂载路径，传入大于128字节的字符数组，例如char mountpoing[129]={0};
返回值
        0 表示获取挂载点成功，1表示未挂载
        其它表示错误,具体参照错误列表
*/
    int crypt_get_mountpoint(char *volName, char *mountpoint);

/* 
返回占用文件列表
输入参数
		devpath: 设备挂载点
输出参数
	    文件的个数
返回值
        文件列表，NULL表示获取失败
示例：
    int ret = 0;
	char **filelist;
	filelist = get_all_file_status("/media/jxs/", &ret);
	for (int i= 0; i<ret; i++)
	{
		printf("filelist[%d]: %s\n", i, filelist[i]);
	}

	for (int j= 0; j<ret; j++)
	{
		free(filelist[j]);
	}
	free(filelist);
*/
    char **crypt_get_all_file_status(char *devpath, int *count);

/* 
返回设备是否正在使用磁盘加密工具
输入参数
		devpath: 设备名称
返回值
        -1代表函数报错
        0代表当前设备正在使用磁盘加密工具
        1代表当前设备未使用磁盘加密工具
*/
    int check_device_is_using_diskenc(char *devpath);
/* 
获取映射点
输入参数
		device: 设备路径，例如/dev/sdb1
输出参数
	    path：映射路径，需传入大于128字节的字符数组，例如char path[129]={0};
返回值
        0 表示获取映射点成功，其它表示错误,具体参照错误列表
*/
    int crypt_disk_get_dev_mapper(char *device, char *path);
  

    /*
输入参数：mount_point 挂载点文件路径
        pw_uid：用户id
        pw_gid :组id
返回值：
        0 表示成功
       其它表示错误
*/
    int crypt_disk_change_chown_mountpoint(char *mount_point, uid_t pw_uid, gid_t pw_gid);

    /* 
polkit认证
输入参数
        actid: polkit动作id
输出参数
        无
返回值
        1 认证成功， 0 认证失败
*/ 
    int crypt_disk_authority_check_by_polkit(char *actid);
    /* 
释放共享内存
输入参数
        无
输出参数
        无
返回值
        -1 释放失败， 0 释放成功
*/    
    int delate_shm_for_bar(char *device);   
#ifdef __cplusplus
}
#endif
#endif
